import 'package:flutter/material.dart';
import 'package:flutter_custom_toolbox/flutter_toolbox.dart';

import 'package:momomotus/config/default_game_settings.dart';
import 'package:momomotus/models/settings/settings_game.dart';

part 'settings_game_state.dart';

class GameSettingsCubit extends HydratedCubit<GameSettingsState> {
  GameSettingsCubit() : super(GameSettingsState(settings: GameSettings.createDefault()));

  void setValues({
    String? lang,
    String? length,
    String? level,
  }) {
    emit(
      GameSettingsState(
        settings: GameSettings(
          lang: lang ?? state.settings.lang,
          length: length ?? state.settings.length,
          level: level ?? state.settings.level,
        ),
      ),
    );
  }

  String getParameterValue(String code) {
    switch (code) {
      case DefaultGameSettings.parameterCodeLang:
        return GameSettings.getLangValueFromUnsafe(state.settings.lang);
      case DefaultGameSettings.parameterCodeLength:
        return GameSettings.getLengthValueFromUnsafe(state.settings.length);
      case DefaultGameSettings.parameterCodeLevel:
        return GameSettings.getLevelValueFromUnsafe(state.settings.level);
    }

    return '';
  }

  void setParameterValue(String code, String value) {
    final String lang = code == DefaultGameSettings.parameterCodeLang
        ? value
        : getParameterValue(DefaultGameSettings.parameterCodeLang);
    final String length = code == DefaultGameSettings.parameterCodeLength
        ? value
        : getParameterValue(DefaultGameSettings.parameterCodeLength);
    final String level = code == DefaultGameSettings.parameterCodeLevel
        ? value
        : getParameterValue(DefaultGameSettings.parameterCodeLevel);

    setValues(
      lang: lang,
      length: length,
      level: level,
    );
  }

  @override
  GameSettingsState? fromJson(Map<String, dynamic> json) {
    final String lang = json[DefaultGameSettings.parameterCodeLang] as String;
    final String length = json[DefaultGameSettings.parameterCodeLength] as String;
    final String level = json[DefaultGameSettings.parameterCodeLevel] as String;

    return GameSettingsState(
      settings: GameSettings(
        lang: lang,
        length: length,
        level: level,
      ),
    );
  }

  @override
  Map<String, dynamic>? toJson(GameSettingsState state) {
    return <String, dynamic>{
      DefaultGameSettings.parameterCodeLang: state.settings.lang,
      DefaultGameSettings.parameterCodeLength: state.settings.length,
      DefaultGameSettings.parameterCodeLevel: state.settings.level,
    };
  }
}
